#include "bscamera.h"
#include "bssceneobject.h"
#include <math.h>
/*************** FUNCTIONS ***************/

FLOAT clamp(FLOAT val, FLOAT min, FLOAT max) {
	//return the value between the min and max
   if (val<min)
      return min;
   else if (val>max)
      return max;
   else return val;
}

/************** CONSTRUCTOR ***************/


BSCamera::BSCamera() {
	lookat = NULL; //clean the lookat object
	ZeroMemory(&pos, sizeof(pos)); //clean the lookat pos
	ZeroMemory(&lookat_pos, sizeof(lookat_pos)); //clean the lookat pos
	up.x = 0; up.y = 0; up.z = 1;
	//I have changed to define the scene during the drawing
	//that allows to use the same camera to many devices and scenes
	//scene = bsscene; //save the scene

	//default attributes
	fov = 45;
	near_dist = 1;
	far_dist = 100;
};

BSCamera::~BSCamera() {
	Release();
};

void BSCamera::Release() {

};

/*************** POSISTION *************/

FLOAT BSCamera::getLookPosX() { if (lookat == NULL) return lookat_pos.x; else return lookat->getPosX(); };
FLOAT BSCamera::getLookPosY() { if (lookat == NULL) return lookat_pos.y; else return lookat->getPosY(); };
FLOAT BSCamera::getLookPosZ() { if (lookat == NULL) return lookat_pos.z; else return lookat->getPosZ(); };
BSPOS BSCamera::getLookPos() { if (lookat == NULL) return lookat_pos; else return lookat->getPos(); };

void BSCamera::adjustDistance(FLOAT distance) {
	if (lookat != NULL) {
		//adjust the posistion to the object (target)
		//it uses the wheel as modifier
		BSPOS newpos = ((pos - getLookPos()) * -(1 - pow((FLOAT) 1.1, distance / 120) ))+pos;
		pos = newpos;
	}
};

void BSCamera::moveAround(FLOAT dx, FLOAT dy) {
	//moving camera around the posistion
	//I can use this function to move objects around each other

	if (lookat != NULL) {
		//this function only works if the camera is looking to an object

		//probably there is a more simple way to do this, maybe changing the D3DMATRIX(0) and using cos and sin to control the position

		//get the diference to know the point
		D3DXVECTOR3 v_dist = BSToVector3(pos - lookat->getPos()); //vector distance
		
		D3DXVECTOR3 v_up = BSToVector3(up); //vector absolute up main axis
		FLOAT dist = D3DXVec3Length(&v_dist); //camera distance to the point = sqrt(v_dist.x * v_dist.x + v_dist.y * v_dist.y + v_dist.z * v_dist.z); 

		//temporary variables to be used as out
		D3DXVECTOR3 rotate;
		D3DXVECTOR3 temp;
		D3DXQUATERNION quat;
		D3DXMATRIX matrix;

		v_dist = *D3DXVec3Normalize(&temp, &v_dist); //normalise the vector


		//normal right will tell the camera's right
		D3DXVECTOR3 normalright = *D3DXVec3Cross(&temp, &v_up, &v_dist);
		normalright = *D3DXVec3Normalize(&temp, &normalright);

		FLOAT pitch = acos(D3DXVec3Dot(&v_up, &v_dist)); //radian of the angle
		pitch = clamp(pitch+(-BSToRadian(dy)), (FLOAT) (0+0.025), (FLOAT) (3.1415-0.025));

		//rotate the vector
		//it transform the normalright
		D3DXVec3TransformCoord(&temp, &v_up, D3DXMatrixRotationQuaternion(&matrix, D3DXQuaternionRotationAxis(&quat, &normalright, pitch)));
		D3DXVec3TransformCoord(&rotate, &temp, D3DXMatrixRotationQuaternion(&matrix, D3DXQuaternionRotationAxis(&quat, &v_up, BSToRadian(dx))));
		rotate = *D3DXVec3Scale(&temp, &rotate, dist); //resizing vector

		//get the new posistion
		D3DXVec3Add(&temp, &BSToVector3(pos), &(rotate - BSToVector3(pos - lookat->getPos())) );

		//it is possible to move the camera smoothly, I would just need to define this new posistion as final destination
		//and in the loop or somewhere else move the camera slowly

		pos.x = temp.x;
		pos.y = temp.y;
		pos.z = temp.z;


	}

};


 

/************** DRAW ***************/

void BSCamera::draw(BSScene* scene) {

	//get the object posistion or the look at posistion
	BSPOS tpos;
	if (lookat == NULL) {
		tpos.x = lookat_pos.x;
		tpos.y = lookat_pos.y;
		tpos.z = lookat_pos.z;
	} else {
		tpos.x = lookat->getPosX();
		tpos.y = lookat->getPosY();
		tpos.z = lookat->getPosZ();
	}
	
	//prepare the matrix with the camera data
    D3DXMatrixLookAtLH(&matView,
                       &D3DXVECTOR3 (pos.x, pos.y, pos.z),    // the camera position
                       &D3DXVECTOR3 (tpos.x, tpos.y, tpos.z),    // the look-at position
                       &D3DXVECTOR3 (up.x, up.y, up.z));    // the up direction

    scene->getDevice()->SetTransform(D3DTS_VIEW, &matView);    // set the view transform to matView

	//get the screen size to do the aspect ratio
	UINT w, h;
	scene->getBSDevice()->getScreenSize(&w, &h);

    D3DXMatrixPerspectiveFovLH(&matProjection,
                               D3DXToRadian(fov),    // the horizontal field of view
							   (FLOAT)w / (FLOAT)h, // aspect ratio
                               near_dist,    // the near view-plane
                               far_dist);    // the far view-plane

    scene->getDevice()->SetTransform(D3DTS_PROJECTION, &matProjection);    // set the projection
};

