/*
  Camera Class
*/
#ifndef _BSCAMERA
#define _BSCAMERA
#include "bstypes.h"

class BSSceneObject;
class BSScene;

class BSCamera {
	private:
		BSSceneObject* lookat; //pointer to object to look at
		BSPOS lookat_pos; //if no object, it uses this position
		BSPOS pos; //camera position
		BSPOS up; //UP posistion (camera rotation/translation)
		FLOAT fov; //angle of field of view
		FLOAT near_dist, far_dist; //distances
		D3DXMATRIX matView;    // the view transform matrix
	    D3DXMATRIX matProjection;     // the projection transform matrix
	public:
		BSCamera(); //constructor
		~BSCamera(); //destructor

		void Release();

		//Setters and Getters to the camera position
		void setPosX(FLOAT value) { pos.x = value; };
	    void setPosY(FLOAT value) { pos.y = value; };
	    void setPosZ(FLOAT value) { pos.z = value; };
		void setPos(BSPOS value) { pos = value; };
		FLOAT getPosX() { return pos.x; };
		FLOAT getPosY() { return pos.y; };
		FLOAT getPosZ() { return pos.z; };
		BSPOS getPos() { return pos; };

		//Setters and Getters to the LOOKAT position
		void setLookPosX(FLOAT value) { lookat_pos.x = value; };
	    void setLookPosY(FLOAT value) { lookat_pos.y = value; };
	    void setLookPosZ(FLOAT value) { lookat_pos.z = value; };
		void setLookPos(BSPOS value) { lookat_pos = value; };
		FLOAT getLookPosX();
		FLOAT getLookPosY();
		FLOAT getLookPosZ();
		BSPOS getLookPos(); //show the object posistion or vector

		//Setters and Getters to the UP position
		void setUpX(FLOAT value) { up.x = value; };
	    void setUpY(FLOAT value) { up.y = value; };
	    void setUpZ(FLOAT value) { up.z = value; };
		void setUp(BSPOS value) { up = value; };
		FLOAT getUpX() { return up.x; };
		FLOAT getUpY() { return up.y; };
		FLOAT getUpZ() { return up.z; };
		BSPOS getUp() { return up; };

		//FoV and Distance Setters and Getters
		void setFov(FLOAT value) { fov = value; };
		void setNearDist(FLOAT value) { near_dist = value; };
		void setFarDist(FLOAT value) { far_dist = value; };
		FLOAT getFov() { return fov; };
		FLOAT getNearDist() { return near_dist; };
		FLOAT getFarDist() { return far_dist; };


		//Define scene object to loot at
		void setLookAt(BSSceneObject* obj) { lookat = obj; };

		//Move around the target
		void moveAround(FLOAT dx, FLOAT dy);
		void adjustDistance(FLOAT distance);

		//Matrixes
		D3DXMATRIX getMatView() { return matView; };  //return the view matrix 
		D3DXMATRIX getMatProj() { return matProjection; };  //return the projection matrix

		//Camera Properties
		void draw(BSScene* scene);
};
#endif

