/**
    Main Class of the Test
  This class create all DirectX devices, the Window and handle/distribute
  all input events.
*/
#include "bsdevice.h"

/************* WINDOW HANDLE ****************/
//a simple window handle just to quit
LRESULT CALLBACK WindowProc(HWND hWnd,
                         UINT message,
                         WPARAM wParam,
						 LPARAM lParam) { if (message == WM_DESTROY) { PostQuitMessage(0); return 0; } return DefWindowProc (hWnd, message, wParam, lParam); };

/************* CONSTRUCTORS ****************/

//BSDevice - Create the basic variables to hold the information
BSDevice::BSDevice(HINSTANCE hInst) {
	hInstance = hInst; //save the application instance

    ZeroMemory(&d3dpp, sizeof(d3dpp));    // clear out the struct for use

	running = FALSE; //set the device as not running

	//defining all directx components as NULL
	d3d = NULL;
	d3ddev = NULL;
	din = NULL;
	dinkeyboard = NULL;
	dinmouse = NULL;

	//defines default parameters to this Test
	//those parameters could be defined by class methods
	fullscreen = FALSE;
	width = 640;
	height = 480;
    d3dpp.Windowed = TRUE;    // program windowed, not fullscreen
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;    // discard old frames
    d3dpp.BackBufferFormat = D3DFMT_X8R8G8B8;    // set the back buffer format to 32-bit
    d3dpp.EnableAutoDepthStencil = TRUE;     //to have depth
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
}

BSDevice::~BSDevice() {
	Release();
}

void BSDevice::Release() {
    if (NULL != d3d) d3d->Release();
	if (NULL != d3ddev) d3ddev->Release();
}

/************* BSDEVICE PROPERTIES ****************/

void BSDevice::setInitialScreenSize(UINT w, UINT h) {
	//I could have another method to define the screen size and then change the device and window
	//properties in runtime
	width = w;
	height = h;
}

void BSDevice::setBackBufferFormat(D3DFORMAT bbf) {
	d3dpp.BackBufferFormat = bbf;
}

void BSDevice::setFullScreen(BOOL fs) {
	//in this function I could change the window and devices parameters to
	//change in run time the window mode, but for now let just focus on the basics
	d3dpp.Windowed = !fs;
	fullscreen = fs;
}

void BSDevice::getScreenSize(UINT* w, UINT* h) {
	*w = width;
	*h = height;
}

BOOL BSDevice::getFullScreen() {
	return fullscreen;
}

/************* BSDEVICE FUNCTIONS ****************/

void BSDevice::quit() {
	running = FALSE;
}

BOOL BSDevice::run() {
	//for now I will do something very simple, set a variable to see if everything
	//was already created or not
	if (!running) {
		//if the device has not been created, it will create now and show the window
		if (d3ddev == NULL) {
			createDevice();
		}
	    ShowWindow(hWnd, TRUE);
		running = !running;
	} else {
		MSG msg;
	    if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
			//verifies all messages just to keep running
			if (msg.message == WM_QUIT)
				running = FALSE;

			//testing to resize window... not working yet
			if (msg.message == WM_SIZE) {
				resizeWindow(LOWORD(msg.lParam), HIWORD(msg.lParam));
			}

			// Translate the message and dispatch it to WindowProc()
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

		doMouseEvents();
		doKeyBoardEvents();
    }
	return running;
}


void BSDevice::beginScene(BSCOLOUR colour) {
	//clear the screen with specified colour
	d3ddev->Clear(0, NULL, D3DCLEAR_TARGET, colour, 1.0f, 0);
    d3ddev->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0);

	//start 3d scene
	d3ddev->BeginScene();
};

void BSDevice::endScene() {
	d3ddev->EndScene();
    d3ddev->Present(NULL, NULL, NULL, NULL);
};

LPDIRECT3DDEVICE9 BSDevice::getDevice() {
	if (d3ddev == NULL) {
		createDevice();
	}
	return d3ddev;
}
void BSDevice::getBackBufferSize(UINT* w, UINT* h) { 
	IDirect3DSurface9* pBackBuffer;
	d3ddev->GetBackBuffer( 0, 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer );
	D3DSURFACE_DESC pDesc;
	pBackBuffer->GetDesc(&pDesc);
	*w = pDesc.Width;
	*h = pDesc.Height;
	pBackBuffer->Release();
};



/************* CREATING DEVICES ****************/

void BSDevice::createWindow(DWORD style) {
    WNDCLASSEX wc;

    // clear out the window class for use
    ZeroMemory(&wc, sizeof(WNDCLASSEX));

    // fill in the struct with the needed information
    wc.cbSize = sizeof(WNDCLASSEX);
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC)WindowProc;
    wc.hInstance = hInstance;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.lpszClassName = "BinaryStar";

    // register the window class
    RegisterClassEx(&wc);

    // create the window and use the result as the handle
    hWnd = CreateWindowEx(NULL,
                          "BinaryStar",    // name of the window class
                          "Binary Star Application",   // title of the window
                          style,    // window style
                          0,    // x-position of the window
                          0,    // y-position of the window
                          width,    // width of the window
                          height,    // height of the window
                          NULL,    // we have no parent window, NULL
                          NULL,    // we aren't using menus, NULL
                          hInstance,    // application handle
                          NULL);    // used with multiple windows, NULL
}


void BSDevice::createDevice() {

	//first create the window
	//I have put the window creation here because we can only create the device
	//if we have a window, and the window is useless without the device
	createWindow(fullscreen ? (WS_EX_TOPMOST | WS_POPUP) : (WS_OVERLAPPEDWINDOW));
	// create the Direct3D interface
    d3d = Direct3DCreate9(D3D_SDK_VERSION);
    
	//define the device window as the previous created window
    d3dpp.hDeviceWindow = hWnd;    // set the window to be used by Direct3D
	d3dpp.Windowed = !fullscreen;
    d3dpp.BackBufferWidth = width;    // set the width of the buffer
    d3dpp.BackBufferHeight = height;    // set the height of the buffer

	// create a device class using this information and information from the d3dpp stuct
    d3d->CreateDevice(D3DADAPTER_DEFAULT,
                      D3DDEVTYPE_HAL,
                      hWnd,
					  D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                      &d3dpp,
                      &d3ddev);

    //I will create a function to change the lights on runtime
	d3ddev->SetRenderState(D3DRS_LIGHTING, TRUE);    // turn on the 3D lighting
    d3ddev->SetRenderState(D3DRS_ZENABLE, TRUE);    // turn on the z-buffer
    d3ddev->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(50, 50, 50));    // ambient light

	//create input devices
    // create the DirectInput interface
    DirectInput8Create(hInstance,    // the handle to the application
                       DIRECTINPUT_VERSION,    // the compatible version
                       IID_IDirectInput8,    // the DirectInput interface version
                       (void**)&din,    // the pointer to the interface
                      NULL);    // COM stuff, for now it will be null

    // create the keyboard device
    din->CreateDevice(GUID_SysKeyboard,    // the default keyboard ID being used
                      &dinkeyboard,    // the pointer to the device interface
                      NULL);    // COM stuff, for now it will be null

    // create the mouse device
    din->CreateDevice(GUID_SysMouse,    // the default mouse ID being used
                      &dinmouse,    // the pointer to the device interface
                      NULL);    // COM stuff, for now it will be null

    dinkeyboard->SetDataFormat(&c_dfDIKeyboard); // set the data format to keyboard format
    dinmouse->SetDataFormat(&c_dfDIMouse);    // set the data format to mouse format

    // set the control you will have over the keyboard
    dinkeyboard->SetCooperativeLevel(hWnd,
                                     DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
    // set the control you will have over the mouse
	//define only to work on the application, we dont need to catch outside movements
    dinmouse->SetCooperativeLevel(hWnd,
		DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

    dinmouse->Acquire();    // Acquire the mouse only once
	

	//creating light - later I will add the functions
    D3DLIGHT9 light;    // create the light struct

    ZeroMemory(&light, sizeof(light));    // clear out the struct for use
    light.Type = D3DLIGHT_DIRECTIONAL;    // make the light type 'directional light'
    light.Diffuse.r = 0.5f;    // .5 red
    light.Diffuse.g = 0.5f;    // .5 green
    light.Diffuse.b = 0.5f;    // .5 blue
    light.Diffuse.a = 1.0f;    // full alpha (we'll get to that soon)

    D3DVECTOR vecDirection = {-1.0f, -0.3f, -1.0f};    // the direction of the light
    light.Direction = vecDirection;    // set the direction

    d3ddev->SetLight(0, &light);    // send the light struct properties to light #0
    d3ddev->LightEnable(0, TRUE);    // turn on light #0

    return;
}

void BSDevice::resizeWindow(LONG w, LONG h) {
	height = h;
	width = w;

	//have to update the backbuffer size

/*	D3DXMATRIX projection_matrix;
	D3DXMatrixPerspectiveFovLH(&projection_matrix, 45.0f, (float)width/(float)height, 0.1f, 100.0f );
	D3DXMatrixIdentity(&projection_matrix);*/

};


/************* DIRECTX EVENTS HANDLE ****************/

void BSDevice::addMouseEvent(MOUSEEVENT me) {
	mouseEvent.insert(mouseEvent.end(), me);
};
void BSDevice::addKeyEvent(KEYBOARDEVENT ke) {
	keyEvent.insert(keyEvent.end(), ke);
};		//add a new keyboard event function
void BSDevice::removeMouseEvent(MOUSEEVENT me) {
	mouseEvent.remove(me);
};	//remove mouse event
void BSDevice::removeKeyEvent(KEYBOARDEVENT ke) {
	keyEvent.remove(ke);
};	//remove keyboard event


void BSDevice::doMouseEvents() {
    static DIMOUSESTATE mousestate;    // create a static storage for the mouse-states

    dinmouse->Acquire();    // get access if we don't have it already

    // fill the mousestate with values
    dinmouse->GetDeviceState(sizeof(DIMOUSESTATE), (LPVOID)&mousestate);

	//saving parameter
	MOUSEDETAIL param;
	param.x = mousestate.lX;
	param.y = mousestate.lY;
	param.z = mousestate.lZ;

	//NOTE: to create mouse hardware I should use mouse exclusive in the device,
	//		create a sprite to be the mouse and variables to control the posistion  (x, y)
	//		the movement would be controled here

	for (UINT i = 0; i <= 3; i++) //moving
		param.bnt[i] = mousestate.rgbButtons[i];

	//sending event to reigstred functions
	MOUSELISTENER::iterator ml;
	for (ml =  mouseEvent.begin(); ml != mouseEvent.end(); ++ml)
		(*ml)(param);

    return;
} 

// this is the function that detects keystrokes and displays them in a message box
void BSDevice::doKeyBoardEvents() {
    static BYTE keystate[256];    // create a static storage for the key-states

    dinkeyboard->Acquire();    // get access if we don't have it already

    dinkeyboard->GetDeviceState(256, (LPVOID)keystate);    // fill keystate with values

	//saving the key in struct to send as parameter
	KEYBOARDDETAIL param;
	for (UINT i = 0; i <= 255; i++) //moving
	  param.key[i] = keystate[i];

	//sending event to reigstred functions
	KEYBOARDLISTENER::iterator kl;
	for (kl =  keyEvent.begin(); kl != keyEvent.end(); ++kl)
		(*kl)(param);

    return;
};
