/**
  BSDevice Class
    This class will be the "heart" of the program. It will have 
  DirectX Devices for drawing and inputs and will make them work.
*/
#ifndef _BSDEVICE

//BS includes
#include "bstypes.h"

// Standard Windows includes
#include <windows.h>
#include <windowsx.h>

// To create list

#include <list>
#include <iostream>

// Direct3D9 includes
#include <d3d9.h>
#include <d3dx9.h>
#include <dinput.h>


// include the DirectX Library files
#pragma comment (lib, "dinput.lib")
#pragma comment (lib, "dinput8.lib")
#pragma comment (lib, "dxguid.lib")
#pragma comment (lib, "d3d9.lib")
#pragma comment (lib, "d3dx9.lib")

using std::list;
using std::allocator;
using std::string;


//event prototype methods defined on bstypes
typedef list< MOUSEEVENT, allocator< MOUSEEVENT> > MOUSELISTENER; //Mouse Listener Type - Container
typedef list< KEYBOARDEVENT, allocator< KEYBOARDEVENT> > KEYBOARDLISTENER; //Keyboard Listener Type - Container


class BSDevice {
	private: 
		MOUSELISTENER mouseEvent;	//container to hold mouse events
		KEYBOARDLISTENER keyEvent;	//container to hold keyboard events


		BOOL running;
		BOOL fullscreen;
		UINT width, height;         // current/initial screen sizes
		HWND hWnd;					// window handle
		HINSTANCE hInstance;		// application instance
		LPDIRECT3D9 d3d;            // the pointer to our Direct3D interface
		LPDIRECT3DDEVICE9 d3ddev;   // the pointer to the device class
		D3DPRESENT_PARAMETERS d3dpp;// create a struct to hold various device information

		LPDIRECTINPUT8 din;					// the pointer to our DirectInput interface
		LPDIRECTINPUTDEVICE8 dinkeyboard;   // the pointer to the keyboard device
		LPDIRECTINPUTDEVICE8 dinmouse;	    // the pointer to the mouse device

		void createWindow(DWORD style);		// function to create the Window
		void createDevice();				// function to create the d3d device

		void doMouseEvents();				//function to capture and distribute mouse events
		void doKeyBoardEvents();			//function to capture and distribute keyboard evets

		void resizeWindow(LONG w, LONG h);

	public:
		BSDevice(HINSTANCE);				// create the object with the application instance
		~BSDevice();						// destroy and release all objects

		void Release();						// release objects

		BOOL run();							// create Windows and DirectX devices and say if it is running properly
		void quit();						// set the running as false to stop the device and finish the loop

		LPDIRECT3DDEVICE9 getDevice();		// function to get the created device

		void setInitialScreenSize(UINT, UINT);//defines initial device and window size
		void setBackBufferFormat(D3DFORMAT);//define device back buffer format
		void setFullScreen(BOOL);			//defines if it is full screen or not
		void getScreenSize(UINT* w, UINT* h);//get the pointers as the screensize
		BOOL getFullScreen();				//get the value of full screen
		void getBackBufferSize(UINT* w, UINT* h);//return the back Buffer Size into the parameters
		HWND getWindowHandle() { return hWnd; }; //return the window handle

		void addMouseEvent(MOUSEEVENT);		//add a new mouse event function
		void addKeyEvent(KEYBOARDEVENT);	//add a new keyboard event function
		void removeMouseEvent(MOUSEEVENT);	//remove mouse event
		void removeKeyEvent(KEYBOARDEVENT);	//remove keyboard event

		void beginScene(BSCOLOUR); //begin the 3D scene with a background colour
		void endScene(); //end the 3D scene

};
#define _BSDEVICE
#endif


