#include "bsmeshx.h"

/***************** CONSTRUCTORS *******************/


//function to create the mesh variables
void BSMeshX::create() {

}
//function to release this mesh special properties
void BSMeshX::Release() {
	BSSceneNod::Release();
}


/***************** LOAD FUNCTIONS *******************/


//function to load the mesh from a file
void BSMeshX::loadFromFile(string file) {
	//clan the loaded mesh before loading a new one
	Release();

	filename = file;


    //loading mesh from file
	D3DXLoadMeshFromX(file.c_str(), D3DXMESH_MANAGED, scene->getDevice(), NULL, &mat, NULL, &numMat, &mesh);

    DWORD *adj = NULL;

    if( !(mesh->GetFVF() & D3DFVF_NORMAL) ) //verify if the mesh has the normals
    {
        ID3DXMesh* tmesh; //temp mesh to be created
        mesh->CloneMeshFVF( mesh->GetOptions(), //create a new mesh with space to light properties
                                  mesh->GetFVF() | D3DFVF_NORMAL, //add the normal in the FVF using the mesh properties
                                  scene->getDevice(), &tmesh );
        D3DXComputeNormals( tmesh, NULL );//create the light normals

		if (mesh != NULL) { mesh->Release(); mesh = NULL; } //clean the old mesh
        mesh = tmesh; //move the new mesh to the object position
    }

	//generating adjacency - mesh edges - to improve performace
	adj = new DWORD[mesh->GetNumFaces() * sizeof(DWORD)];
    mesh->GenerateAdjacency(1e-6f,adj);
    mesh->OptimizeInplace(D3DXMESHOPT_VERTEXCACHE, adj, NULL, NULL, NULL);
    delete []adj;

	//loading materials
	
	//creating texture array
	tex = new LPDIRECT3DTEXTURE9[numMat];
	// retrieve the pointer to the buffer containing the material information
	D3DXMATERIAL* tempMaterials = (D3DXMATERIAL*)mat->GetBufferPointer();

	// create a new material buffer for each material in the mesh
	material = new D3DMATERIAL9[numMat];


	for(DWORD i = 0; i < numMat; i++)    // for each material...
	{
		material[i] = tempMaterials[i].MatD3D;    // get the material info...
		material[i].Ambient = material[i].Diffuse;    // and make ambient the same as diffuse

		if(FAILED(D3DXCreateTextureFromFile(scene->getDevice(), //creating texture
											(LPCSTR) (tempMaterials[i].pTextureFilename), //forcing typecast to get the filename
											&tex[i]))) {
	        tex[i] = NULL;    // if there is no texture, set the texture to NULL
		}
	}

//mesh ready!
};

//drawing mesh
void BSMeshX::draw() {
	//I am not sure where the transformation should be placed (should be before each drawsubset?)
	//I need a model a example with more subsets
	for(DWORD i = 0; i < numMat; i++)    // loop through each subset
	{
		scene->getDevice()->SetMaterial(&material[i]);    // set the appropriate material for the subset
	    if(tex[i] != NULL)    // if the subset has a texture (if texture is not NULL)
	        scene->getDevice()->SetTexture(0, tex[i]);    // ...then set the texture

		scene->getDevice()->SetTransform(D3DTS_WORLD, &matrixTransform()); //rotate, scale and posistion of the mesh according to the nods
		mesh->DrawSubset(i); // draw the subset
	}

	//after drawing the mesh, draw the rest of the nod
	BSSceneNod::draw();
};


/***************** PROPERTIES *******************/

/* just to test the mouse click */

void BSMeshX::changeGreen(FLOAT value) {
	//simple function to change the material colour
	for(DWORD i = 0; i < numMat; i++)    // for each material...
	{
		material[i].Ambient.g = value;
	}
};
FLOAT BSMeshX::getGreen() {
	FLOAT result;
	for(DWORD i = 0; i < numMat; i++)    // for each material...
	{
		result = material[i].Ambient.g;
	}
	return result;
};