#include "bsscene.h"
#include "bscamera.h"
#include "bssceneobject.h"

/************** CONSTRUCTORS ***************/

BSScene::BSScene(BSDevice* bsdev) {
	bsd = bsdev;

	camera = NULL; //no camera, yet
}

BSScene::~BSScene() {
	Release();
}

void BSScene::Release() {
	//release all objects of the scene
	BSSCENEOBJECTLIST::iterator obj;
	BSSCENEOBJECTLIST::iterator next;
	for (next = getListBegin(); next != getListEnd(); ) {
		obj = next;			//define the object to release as next
		++next;				//get the next before removing it
		removeObject(*obj, TRUE);	//delete from the list and destroy it
		//delete *obj;
	}
};

/************** CAMERA ***************/


BSCamera* BSScene::addNewCamera() {
	//create a new blank camera in this scene
	BSCamera* cam = new BSCamera();
	//add the camera to the scene
	addCamera(cam);
	//if no camera defined, use this one
	if (camera == NULL) camera = cam;
	return cam;
}; //function to create a new camera and add into the list

void BSScene::addCamera(BSCamera* cam) {
	camlist.insert( camlist.begin(), cam);
}; //function to add an already created camera

void BSScene::removeCamera(BSCamera* cam, BOOL destroy) {
	camlist.remove( cam);
	if (destroy) cam->Release();
}; //remove a camera from the list and, if needed, destroy it


/************** DRAW ***************/

void BSScene::drawAll() {
	drawAllObjects();
	if (camera != NULL) camera->draw(this);
};

void BSScene::drawAllObjects() {
	//search and draw all objects of the scene
	BSSCENEOBJ obj;
	for (obj =  getListBegin(); obj != getListEnd(); ++obj) {
		(*obj)->draw();
	}
};
