/**
  BSScene Class
    This class will be the "torso" of the application. It will hold
  every single 3D object and the camera.
*/
#ifndef _BSSCENE
#define _BSSCENE
#include "bsdevice.h"
#include "bstypes.h"
#include "bsscenecontainer.h"

class BSSceneObject;
class BSCamera;

typedef list< BSCamera*, allocator< BSCamera*> > BSSCENECAMERALIST; //List of cameras
typedef BSSCENEOBJECTLIST::iterator BSSCENEOBJ; //unsafe basic type of iterator object (to use with begin and end)

class BSScene : public BSSceneContainer {
	private:
		BSDevice* bsd;   //the pointer to the bsdevice (if the pointer of directx device changes, it wont broke the scene)
		BSSCENECAMERALIST camlist; //list of all cameras

		BSCamera* camera;

	public:
		BSScene(BSDevice*);
		~BSScene();

		BSDevice* getBSDevice() { return bsd; }; // return the device pointer to be used on the 
		LPDIRECT3DDEVICE9 getDevice() { return bsd->getDevice(); }; // return the device pointer to be used on the 

		void Release();	//release all objects


		BSCamera* addNewCamera(); //function to create a new camera and add into the list
		void addCamera(BSCamera* cam); //function to add an already created camera
		void removeCamera(BSCamera* cam, BOOL destroy); //remove a camera from the list and, if needed, destroy it
		BSCamera* getCamera() { return camera; };

		void drawAll(); //function to draw all objects in the scene list including the camera
		void drawAllObjects(); //function to draw all objects in the scene list including the camera

		//overriding the protected members of the container
		void addObject(BSSceneObject* obj) { BSSceneContainer::addObject(obj); }; //function to add a new object in the scene list
		void removeObject(BSSceneObject* obj, BOOL destroy) { BSSceneContainer::removeObject(obj, destroy); } ; //function to remove an object from the scene list
		BSSCENEOBJ getListBegin() { return BSSceneContainer::getListBegin(); }
		BSSCENEOBJ getListEnd() { return BSSceneContainer::getListEnd(); }

};
#endif