/* Basic CLass Structure to be Used on the Scene Objects */
#ifndef _BSSCENEOBJECT
#include "bstypes.h"
#include "bsscene.h"

class BSSceneObject {
	private:
		D3DXMATRIX m_rotX;
		D3DXMATRIX m_rotY;
		D3DXMATRIX m_rotZ;
		D3DXMATRIX m_position;
		D3DXMATRIX m_scale;
	protected:
		BSScene* scene;		//scene where is the object to get the device
		BSPOS pos;			//scene basic position
		BSPOS rot;			//rotation values
		BSPOS scale;		//scale values
		BSSceneObject* parent;
	public:
		BSSceneObject(BSScene* s) { 
			parent = NULL;
			scene = s; ZeroMemory(&pos, sizeof(pos)); ZeroMemory(&rot, sizeof(rot)); scale.x = 1; scale.y = 1; scale.z = 1;
		}; //basic creator 

		BSScene* getScene() { return scene; }; //get Scene

		virtual void draw() = 0; //abstract draw function
		virtual void Release() = 0; //abstract Release function

		//Parent
		BSSceneObject* getParent() { return parent; };
		//to draw the childrens correctly
		BSPOS getDrawPos()		{ if (parent != NULL) { return (parent->getDrawPos() + getPos()); } else { return getPos(); } };
		FLOAT getDrawRotX()		{ if (parent != NULL) { return (parent->getDrawRotX() + getAngleX()); } else { return getAngleX(); } };
		FLOAT getDrawRotY()		{ if (parent != NULL) { return (parent->getDrawRotY() + getAngleY()); } else { return getAngleY(); } };
		FLOAT getDrawRotZ()		{ if (parent != NULL) { return (parent->getDrawRotZ() + getAngleZ()); } else { return getAngleZ(); } };
		BSPOS getDrawScale()	{ if (parent != NULL) { return (parent->getDrawScale() + getScale()); } else { return getScale(); } };


		//Position seters and geters
		void setPosX(FLOAT value) { pos.x = value; };
	    void setPosY(FLOAT value) { pos.y = value; };
	    void setPosZ(FLOAT value) { pos.z = value; };
		void setPos(BSPOS value)  { pos = value; };
		FLOAT getPosX() { return pos.x; };
		FLOAT getPosY() { return pos.y; };
		FLOAT getPosZ() { return pos.z; };
		BSPOS getPos() { return pos; };
		D3DXVECTOR3 getPosVector() { return D3DXVECTOR3 ( pos.x, pos.y, pos.z); };

		//Rotation Angle setters and getters
		//I would create another for radians, but I will only use this one
		void setAngleX(FLOAT angle)  { rot.x = angle; };
		void setAngleY(FLOAT angle)  { rot.y = angle; };
		void setAngleZ(FLOAT angle)  { rot.z = angle; };
		void rollAngleX(FLOAT angle) { rot.x += angle; };
		void rollAngleY(FLOAT angle) { rot.y += angle; };
		void rollAngleZ(FLOAT angle) { rot.z += angle; };
		FLOAT getAngleX() { return rot.x; };
		FLOAT getAngleY() { return rot.y; };
		FLOAT getAngleZ() { return rot.z; };

		//Scale setters and getters
		void setScaleX(FLOAT value)  { scale.x = value; };
		void setScaleY(FLOAT value)  { scale.y = value; };
		void setScaleZ(FLOAT value)  { scale.z = value; };
		void setScale(BSPOS value) { scale = value; };
		FLOAT getScaleX() { return scale.x; };
		FLOAT getScaleY() { return scale.y; };
		FLOAT getScaleZ() { return scale.z; };
		BSPOS getScale() { return scale; };

		//Matrixes
		D3DXMATRIX matrixPos() ; //create the Translation Matrix
		D3DXMATRIX matrixRotationX(); //create the RotationX Matrix
		D3DXMATRIX matrixRotationY(); //create the RotationX Matrix
		D3DXMATRIX matrixRotationZ(); //create the RotationX Matrix
		D3DXMATRIX matrixScale(); //create scale matrix

		//Matrix Groups
		//can improve performace by only creating matrixes when something has changed
		D3DXMATRIX matrixRotation(); //create a matrix with all rotation values
		D3DXMATRIX matrixTransform(); //create the Transform matrix
};
#define _BSSCENEOBJECT
#endif