#include "main.h"

using namespace BSTest;

//Function to handle with the Engine Mouse Movement
void MouseMouseEvent(MOUSEDETAIL mouse) {
	mousepos->setText("Mouse: x="+to_string(mouse.x)+" y="+to_string(mouse.y)+" z="+to_string(mouse.z));
	if (mouse.bnt[1] & 0x80) {
		//tells the camera to move around the looking point using the mouse delta
		cam->moveAround((FLOAT) mouse.x, (FLOAT) mouse.y);
	}
	if (mouse.z != 0) {
		cam->adjustDistance((FLOAT) mouse.z);
	}
	return;
};

//Function to handle with the Engine Mouse Movement
void KeyboardEvent(KEYBOARDDETAIL k) {
	FLOAT changex = 0;
	FLOAT changez = 0;
	FLOAT zoom = 0;
	if (k.key[DIK_ESCAPE] & 0x80) { bsd->quit(); }
	if (k.key[DIK_UP] & 0x80) { changez += KEYBOARD_ROTATION_SPEED; }
	if (k.key[DIK_DOWN] & 0x80) {	changez -= KEYBOARD_ROTATION_SPEED; }
	if (k.key[DIK_LEFT] & 0x80) {	changex -= KEYBOARD_ROTATION_SPEED; }
	if (k.key[DIK_RIGHT] & 0x80) { changex += KEYBOARD_ROTATION_SPEED; }
	if (changex != 0 || changez != 0) {
		cam->moveAround(changex, changez);
	}
	if (k.key[DIK_ADD] & 0x80 || k.key[DIK_EQUALS] & 0x80) { zoom += KEYBOARD_ZOOM_SPEED; }
	if (k.key[DIK_SUBTRACT] & 0x80 || k.key[DIK_MINUS] & 0x80) { zoom -= KEYBOARD_ZOOM_SPEED; }
	if (zoom != 0) {
		cam->adjustDistance(zoom);
	}

};

int WINAPI WinMain(HINSTANCE hInstance,
				   HINSTANCE hPrevInstance,
				   LPSTR lpCmdLine,
				   int nCmdShow)
{
	//creating engine device
	bsd = new BSDevice(hInstance);

	//bsd->setFullScreen(TRUE);				//to run in fullscreen
	bsd->setInitialScreenSize(800, 600);
	bsd->setBackBufferFormat(D3DFMT_X8R8G8B8);
	bsd->addMouseEvent(&MouseMouseEvent);	//adding mouse movement event in the device
	bsd->addKeyEvent(&KeyboardEvent);		//adding keyboard event in the device

	//creating GUI componenets
	bsg = new BSGUI(bsd);									//creating GUI device

	fontArial = new BSFont(bsg, NULL, 25, "Arial");			//creating font test

	mousepos = new BSLabel(NULL, fontArial, "", 0xffffff);	//creating mouselabel
	mousepos->setText("Teste "+to_string(20));
	RECT r = {0,0,500,500};
	mousepos->setRect(r);
	mousepos->setColour(BSCOLOUR_RBG(255,255,255));

	//add objects to the GUI device
	bsg->addObject(fontArial); //no need to add the font
	bsg->addObject(mousepos);

	//creating scene
	scene = new BSScene(bsd);						//create scene device

	mesh = new BSMeshX(scene, NULL);				//create a .X file mesh
	mesh->loadFromFile("Tiny.x");					//load it

	mesh2 = new BSMeshX(scene, mesh);				//second mesh to test posistion and use the first one as parent
	mesh2->loadFromFile("Tiny.x");
	mesh2->setScaleX(0.5);
	mesh2->setScaleY(0.5);
	mesh2->setScaleZ(0.5);
	mesh2->setPosX(100);								//   the position of the mesh will be relative when
	mesh2->setPosY(100);								//   it has a parent defined
	mesh->addObject(mesh2);								//add the mesh2 into the mesh, just to know when to draw (can be here or in the scene)

	scene->addObject(mesh);						//add the meshes in the scene
//	scene->addObject(mesh2);						// no need to to add the mesh2 in the scene, it is on the mesh1
	
	cam = scene->addNewCamera();					//create a new camera and define it as default
	cam->setPosZ(500);
	cam->setPosY(500);
	cam->setFarDist(10000);
	cam->setLookAt(mesh);							//where the camera will be looking at

	while (bsd->run()) {
		bsd->beginScene(BSCOLOUR_RBG(0, 0, 0));		//start scene

//		mesh->rollAngleZ(1);							//roll the mesh to test (mesh and mesh2 will roll)

		scene->drawAll();							//draw all objects including the camera

		bsg->beginText();							//start the sprite text

		bsg->drawAll();								//draw gui components

		bsg->endText();								//end the sprite text

		bsd->endScene();							//end the scene and present the frame
	} //main loop

	//release all objects
	scene->Release();	//including scene objects and nods
	bsg->Release();		//including gui objects
	bsd->Release();		//devices
	return 0;
};

